/*  DO NOT EDIT THIS FILE.

    It has been auto-edited by fixincludes from:

	"fixinc/tests/inc/rpc/xdr.h"

    This had to be done to correct non-standard usages in the
    original, manufacturer supplied header file.  */



#if defined( STRUCT_FILE_CHECK )
struct __file_s;
extern void xdrstdio_create( struct __file_s* );
#endif  /* STRUCT_FILE_CHECK */
