DEFINE_DIAGNOSTIC_KIND (DK_FATAL, "Fatal error: ")
DEFINE_DIAGNOSTIC_KIND (DK_ICE, "Internal compiler error: ")
DEFINE_DIAGNOSTIC_KIND (DK_SORRY, "Sorry, unimplemented: ")
DEFINE_DIAGNOSTIC_KIND (DK_ERROR, "Error: ")
DEFINE_DIAGNOSTIC_KIND (DK_WARNING, "Warning: ")
DEFINE_DIAGNOSTIC_KIND (DK_ANACHRONISM, "Anachronism: ")
DEFINE_DIAGNOSTIC_KIND (DK_NOTE, "Note: ")
